package com.abi.qms.platform.service.impl;

import cn.hutool.core.collection.CollectionUtil;
import com.abi.qms.platform.dao.entity.*;
import com.abi.qms.platform.dao.enums.ActiveAuditStatusEnum;
import com.abi.qms.platform.dao.enums.ActiveModifyTypeEnum;
import com.abi.qms.platform.dao.enums.QrPackageApplyStatusEnum;
import com.abi.qms.platform.dao.enums.ValidEnum;
import com.abi.qms.platform.dao.mapper.*;
import com.abi.qms.platform.dao.vo.result.ActiveModifyApplyPackageVO;
import com.abi.qms.platform.dao.vo.result.ActivePackageVO;
import com.abi.qms.platform.dao.vo.result.ListActiveVO;
import com.abi.qms.platform.dao.vo.result.PackageActiveSearchVO;
import com.abi.qms.platform.dto.req.*;
import com.abi.qms.platform.dto.res.*;
import com.abi.qms.platform.infrastructure.util.AssertUtil;
import com.abi.qms.platform.infrastructure.util.PageUtil;
import com.abi.qms.platform.infrastructure.util.UserUtil;
import com.abi.qms.platform.service.ActiveService;
import com.abi.task.common.api.exception.BusinessException;
import com.abi.task.common.utils.PojoConverterUtils;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.time.LocalDateTime;
import java.util.*;
import java.util.stream.Collectors;

/**
 * <p>
 * 活动
 * </p>
 *
 * @author Andy.Tan
 */
@Service
public class ActiveServiceImpl implements ActiveService {

    @Autowired
    private BaseActiveMapper baseActiveMapper;

    @Autowired
    private BaseActiveQrPackageMappingMapper baseActiveQrPackageMappingMapper;

    @Autowired
    private UserUtil userUtil;

    @Autowired
    private BaseBrandMapper baseBrandMapper;

    @Autowired
    private QrPackageMapper qrPackageMapper;

    @Autowired
    private BaseActiveModifyApplyMapper baseActiveModifyApplyMapper;

    @Autowired
    private BaseActiveModifyApplyQrPackageMappingMapper baseActiveModifyApplyQrPackageMappingMapper;
    
    /**
     * 新增活动
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public void addActive(AddActiveReq req){
        //校验入参
        //活动ID不可重复创建。
        QueryWrapper<BaseActive> activeVali = new QueryWrapper<>();
        activeVali.eq("active_code", req.getActiveCode());
        activeVali.eq("is_delete", 0);
        BaseActive baseActive = baseActiveMapper.selectOne(activeVali);
        if (Objects.nonNull(baseActive)){
            throw new BusinessException("码活动ID" + req.getActiveCode() + "已存在。");
        }

        //1-新增
        Long userId = userUtil.getUser().getId();
        BaseActive active = new BaseActive();
        active.setActiveCode(req.getActiveCode());
        active.setActiveName(req.getActiveName());
        active.setActiveType(req.getActiveType());
        active.setAuditStatus(ActiveAuditStatusEnum.WAIT_REVIEW.getCode());
        active.setApplyTime(LocalDateTime.now());
        active.setApplyUserId(userId);
        active.setApplyUserName(userUtil.getUser().getUserName());
        active.setBeginTime(req.getBeginTime());
        active.setEndTime(req.getEndTime());
        active.setBrandCode(CollectionUtil.isNotEmpty(req.getBrandCodeList()) ? String.join(",", req.getBrandCodeList()) : null);
        active.setActiveUrl(req.getActiveUrl());
        active.setCreateBy(userId);
        active.setUpdateBy(userId);
        baseActiveMapper.insert(active);

        //添加码活动和码包的关联
        List<QrPackage> qrPackages = searchReviewPassPackageByPackageIds(req.getQrPackageIdList());
        List<Long> qrPackageIdsAdd = qrPackages.stream().map(QrPackage::getId).collect(Collectors.toList());
        if(CollectionUtil.isNotEmpty(qrPackageIdsAdd)) {
            baseActiveQrPackageMappingMapper.batchInsert(active.getId(), qrPackageIdsAdd, userId);
        }

    }

    /**
     * 编辑活动
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public void updateActive(UpdateActiveReq req){

        //1-查询活动
        BaseActive active = baseActiveMapper.selectById(req.getId());
        AssertUtil.isNull(active,"活动不存在");

        if(ActiveAuditStatusEnum.REVIEW_PASS.is(active.getAuditStatus())){
            throw new BusinessException("该数据已审核通过，请调用码活动申请单的接口。");
        }

        //2-修改活动
        Long userId = userUtil.getUser().getId();
        active.setActiveName(req.getActiveName());
        active.setActiveType(req.getActiveType());
        //只要经过编辑，就恢复成待审核状态
        active.setAuditStatus(ActiveAuditStatusEnum.WAIT_REVIEW.getCode());
        active.setApplyTime(LocalDateTime.now());
        active.setApplyUserId(userId);
        active.setApplyUserName(userUtil.getUser().getUserName());
        active.setBeginTime(req.getBeginTime());
        active.setEndTime(req.getEndTime());
        active.setBrandCode(CollectionUtil.isNotEmpty(req.getBrandCodeList()) ? String.join(",", req.getBrandCodeList()) : null);
        active.setActiveUrl(req.getActiveUrl());
        active.setUpdateBy(userId);
        baseActiveMapper.updateById(active);


        //删除添加关联的码包，重新插入
        updateActivePackageMappiing(req.getId(), req.getQrPackageIdAddList(), req.getQrPackageIdDeleteList());
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void resubmitActive(ResubmitActiveReq req) {
        //1-查询活动
        BaseActive active = baseActiveMapper.selectById(req.getActiveId());
        AssertUtil.isNull(active,"活动不存在");

        if(!ActiveAuditStatusEnum.REVIEW_PASS.is(active.getAuditStatus())){
            throw new BusinessException("该数据没有审核通过，请调用编辑码活动的接口。");
        }

        //删除原有申请单数据
        UpdateWrapper<BaseActiveModifyApply> basmaUpdate = new UpdateWrapper<>();
        basmaUpdate.set("is_delete", 1)
                .eq("active_id", req.getActiveId());
        baseActiveModifyApplyMapper.update(null, basmaUpdate);
        UpdateWrapper<BaseActiveModifyApplyQrPackageMapping> basmamUpdate = new UpdateWrapper<>();
        basmamUpdate.set("is_delete", 1)
                .eq("active_id", req.getActiveId());
        baseActiveModifyApplyQrPackageMappingMapper.update(null, basmamUpdate);

        //新增活动申请单
        Long userId = userUtil.getUser().getId();
        BaseActiveModifyApply baseActiveModifyApply = new BaseActiveModifyApply();
        baseActiveModifyApply.setActiveId(req.getActiveId());
        baseActiveModifyApply.setActiveName(req.getActiveName());
        baseActiveModifyApply.setActiveType(req.getActiveType());
        baseActiveModifyApply.setAuditStatus(ActiveAuditStatusEnum.WAIT_REVIEW.getCode());
        baseActiveModifyApply.setApplyTime(LocalDateTime.now());
        baseActiveModifyApply.setApplyUserId(userId);
        baseActiveModifyApply.setApplyUserName(userUtil.getUser().getUserName());
        baseActiveModifyApply.setBeginTime(req.getBeginTime());
        baseActiveModifyApply.setEndTime(req.getEndTime());
        baseActiveModifyApply.setBrandCode(CollectionUtil.isNotEmpty(req.getBrandCodeList()) ? String.join(",", req.getBrandCodeList()) : null);
        baseActiveModifyApply.setActiveUrl(req.getActiveUrl());
        baseActiveModifyApply.setUpdateBy(userId);
        baseActiveModifyApply.setCreateBy(userId);
        baseActiveModifyApplyMapper.insert(baseActiveModifyApply);

        //修改码活动是否有申请单
        UpdateWrapper<BaseActive> activeUpdateWrapper = new UpdateWrapper<>();
        activeUpdateWrapper.set("is_modify_apply", 1)
                .eq("id", req.getActiveId());
        baseActiveMapper.update(null, activeUpdateWrapper);

        //批量插入 活动编辑申请码包关联子表
        batchInsertActiveModifyApplyPackageMappiing(baseActiveModifyApply.getId(), req.getActiveId(), req.getQrPackageIdAddList(), req.getQrPackageIdDeleteList());
    }

    /**
     * 分页查询活动
     */
    @Override
    public ListActiveRes listActive(ListActiveReq req){

        //分页查询
        IPage<ListActiveVO> iPage = baseActiveMapper.listActivePage(PageUtil.createPage(req), req);
        List<ListActiveVO> activeList = iPage.getRecords();
        //封装出参
        ListActiveRes res = new ListActiveRes();
        //放入分页信息
        PageUtil.copyPageInfo(res, iPage);
        //放入出参列表
        List<ListActiveRes.ActiveBean> activeBeanList = PojoConverterUtils.copyList(activeList, ListActiveRes.ActiveBean.class);
        res.setActiveBeanList(activeBeanList);

        return res;

    }

    /**
     * 查询活动明细
     */
    @Override
    public GetActiveRes getActive(GetActiveReq req){

        //1-查询活动对象
        BaseActive active = baseActiveMapper.selectById(req.getId());
        AssertUtil.isNull(active, "活动不存在");

        //转化出参
        GetActiveRes res = PojoConverterUtils.copy(active, GetActiveRes.class);
        //处理品牌code，品牌名称
        if(Objects.nonNull(active.getBrandCode())){
            String[] brandSplit = active.getBrandCode().trim().split(",");
            res.setBrandCodeList(Arrays.asList(brandSplit));
            QueryWrapper<BaseBrand> baseBrandQW = new QueryWrapper<>();
            baseBrandQW.in("brand_code", brandSplit);
            List<BaseBrand> baseBrands = baseBrandMapper.selectList(baseBrandQW);
            List<String> baseBrandNameList = baseBrands.stream().map(BaseBrand::getBrandName).collect(Collectors.toList());
            res.setBrandNameList(baseBrandNameList);
        }

        //查询码活动关联的码包列表
        res.setListActivePackageRes(listActivePackage(req));
        return res;

    }

    @Override
    public ListActivePackageRes listActivePackage(GetActiveReq req) {
        //查询码活动关联的码包列表
        IPage<ActivePackageVO> activePackageVOIPage = qrPackageMapper.listPackageById(PageUtil.createPage(req), req.getId());
        List<ActivePackageVO> records = activePackageVOIPage.getRecords();
        ListActivePackageRes listActivePackageRes = new ListActivePackageRes();
        PageUtil.copyPageInfo(listActivePackageRes, activePackageVOIPage);
        List<ListActivePackageRes.PackageBean> packageBeans = PojoConverterUtils.copyList(records, ListActivePackageRes.PackageBean.class);
        listActivePackageRes.setPackageBeanList(packageBeans);
        return listActivePackageRes;
    }

    @Override
    public GetActiveModifyApplyRes getActiveModifyApplyRes(GetActiveModifyApplyReq req) {
        //1-查询活动对象
        BaseActive active = baseActiveMapper.selectById(req.getId());
        AssertUtil.isNull(active, "活动不存在");

        //查询码活动申请单
        QueryWrapper<BaseActiveModifyApply> activeModifyApplyQW = new QueryWrapper<>();
        activeModifyApplyQW.eq("active_id", req.getId());
        activeModifyApplyQW.eq("is_delete", 0);
        BaseActiveModifyApply activeModifyApply = baseActiveModifyApplyMapper.selectOne(activeModifyApplyQW);
        AssertUtil.isNull(activeModifyApply,"码活动申请单不存在");

        GetActiveModifyApplyRes res = new GetActiveModifyApplyRes();

        //转化出参
        GetActiveModifyApplyRes.Active resActive = PojoConverterUtils.copy(active, GetActiveModifyApplyRes.Active.class);
        GetActiveModifyApplyRes.ActiveModifyApply resActiveModifyApply = PojoConverterUtils.copy(activeModifyApply, GetActiveModifyApplyRes.ActiveModifyApply.class);
        //处理品牌code，品牌名称
        if(Objects.nonNull(active.getBrandCode())){
            String[] brandSplit = active.getBrandCode().trim().split(",");
            resActive.setBrandCodeList(Arrays.asList(brandSplit));
            QueryWrapper<BaseBrand> baseBrandQW = new QueryWrapper<>();
            baseBrandQW.in("brand_code", brandSplit);
            List<BaseBrand> baseBrands = baseBrandMapper.selectList(baseBrandQW);
            List<String> baseBrandNameList = baseBrands.stream().map(BaseBrand::getBrandName).collect(Collectors.toList());
            resActive.setBrandNameList(baseBrandNameList);
        }

        if(Objects.nonNull(activeModifyApply.getBrandCode())){
            String[] brandSplit = activeModifyApply.getBrandCode().trim().split(",");
            resActiveModifyApply.setBrandCodeList(Arrays.asList(brandSplit));
            QueryWrapper<BaseBrand> baseBrandQW = new QueryWrapper<>();
            baseBrandQW.in("brand_code", brandSplit);
            List<BaseBrand> baseBrands = baseBrandMapper.selectList(baseBrandQW);
            List<String> baseBrandNameList = baseBrands.stream().map(BaseBrand::getBrandName).collect(Collectors.toList());
            resActiveModifyApply.setBrandNameList(baseBrandNameList);
        }

        res.setActive(resActive);
        res.setActiveModifyApply(resActiveModifyApply);
        //查询码活动关联的码包列表
        res.setListActiveModifyApplyPackageRes(listActiveModifyApplyPackage(req));
        return res;
    }

    @Override
    public ListActiveModifyApplyPackageRes listActiveModifyApplyPackage(GetActiveModifyApplyReq req) {
        //查询码活动关联的码包列表
        IPage<ActiveModifyApplyPackageVO> activeModifyApplyPackageVOIPage = qrPackageMapper.listPackageByActiveApplyId(PageUtil.createPage(req), req.getId());
        List<ActiveModifyApplyPackageVO> records = activeModifyApplyPackageVOIPage.getRecords();
        ListActiveModifyApplyPackageRes listActiveModifyApplyPackageRes = new ListActiveModifyApplyPackageRes();
        PageUtil.copyPageInfo(listActiveModifyApplyPackageRes, activeModifyApplyPackageVOIPage);
        List<ListActiveModifyApplyPackageRes.PackageBean> packageBeans = PojoConverterUtils.copyList(records, ListActiveModifyApplyPackageRes.PackageBean.class);
        listActiveModifyApplyPackageRes.setPackageBeanList(packageBeans);
        return listActiveModifyApplyPackageRes;
    }

    @Override
    public ListActivePackageAddRes listActivePackageAdd(ListActivePackageAddReq req) {
        //查询码活动关联的码包列表
        IPage<PackageActiveSearchVO> packageActiveSearchVOIPage = qrPackageMapper.listPackageByBrandCode(PageUtil.createPage(req), req);
        List<PackageActiveSearchVO> records = packageActiveSearchVOIPage.getRecords();
        ListActivePackageAddRes listActivePackageAddRes = new ListActivePackageAddRes();
        PageUtil.copyPageInfo(listActivePackageAddRes, packageActiveSearchVOIPage);
        List<ListActivePackageAddRes.PackageBean> packageBeans = PojoConverterUtils.copyList(records, ListActivePackageAddRes.PackageBean.class);
        listActivePackageAddRes.setPackageBeanList(packageBeans);
        return listActivePackageAddRes;
    }

    /**
     * 活动启用
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public void enableActive(EnableActiveReq req){
        List<Long> ids = req.getIds();

        //循环启用
        for (Long id : ids) {
            //1-查询活动对象
            BaseActive active = baseActiveMapper.selectById(id);
            AssertUtil.isNull(active, "活动不存在");

            if (!ValidEnum.NOT_VALID.is(active.getValid())) {
                continue;
            }

            //2-修改状态
            active.setValid(ValidEnum.VALID.getCode());
            baseActiveMapper.updateById(active);
        }
    }

    /**
     * 活动禁用
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public void disableActive(DisableActiveReq req){
        List<Long> ids = req.getIds();

        //循环禁用
        for (Long id : ids) {
            //1-查询活动对象
            BaseActive active = baseActiveMapper.selectById(id);
            AssertUtil.isNull(active, "活动不存在");

            if (!ValidEnum.VALID.is(active.getValid())) {
                return;
            }

            //2-修改状态
            active.setValid(ValidEnum.NOT_VALID.getCode());
            baseActiveMapper.updateById(active);
        }
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void passActive(PassActiveReq req) {
        BaseActive active = baseActiveMapper.selectById(req.getId());
        AssertUtil.isNull(active, "活动不存在");

        if(ActiveAuditStatusEnum.WAIT_REVIEW.is(active.getAuditStatus())){
            throw new BusinessException("该活动不能审核");
        }
        Long userId = userUtil.getUser().getId();
        active.setAuditStatus(ActiveAuditStatusEnum.REVIEW_PASS.getCode());
        active.setAuditUserId(userId);
        active.setAuditUserName(userUtil.getUser().getUserName());
        active.setUpdateBy(userId);
        baseActiveMapper.updateById(active);

    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void refuseActive(RefuseActiveReq req) {
        BaseActive active = baseActiveMapper.selectById(req.getId());
        AssertUtil.isNull(active, "活动不存在");
        if(ActiveAuditStatusEnum.WAIT_REVIEW.is(active.getAuditStatus())){
            throw new BusinessException("该活动不能审核");
        }
        Long userId = userUtil.getUser().getId();
        active.setAuditStatus(ActiveAuditStatusEnum.REVIEW_REFUSE.getCode());
        active.setAuditUserId(userId);
        active.setAuditUserName(userUtil.getUser().getUserName());
        active.setUpdateBy(userId);
        baseActiveMapper.updateById(active);

    }

    @Override
    public void passActiveModifyApply(PassActiveModifyApplyReq req) {

        //1-查询活动
        BaseActive active = baseActiveMapper.selectById(req.getActiveId());
        AssertUtil.isNull(active,"活动不存在");

        if(!ActiveAuditStatusEnum.REVIEW_PASS.is(active.getAuditStatus())){
            throw new BusinessException("该数据没有审核通过，请调用审核码活动的接口。");
        }
        //查询码活动申请单
        BaseActiveModifyApply activeModifyApply = baseActiveModifyApplyMapper.selectById(req.getApplyId());
        AssertUtil.isNull(activeModifyApply,"码活动申请单不存在");
        if(ActiveAuditStatusEnum.WAIT_REVIEW.is(activeModifyApply.getAuditStatus())){
            throw new BusinessException("该码活动申请单不能审核");
        }

        //审核通过码活动申请单
        Long userId = userUtil.getUser().getId();
        activeModifyApply.setAuditStatus(ActiveAuditStatusEnum.REVIEW_PASS.getCode());
        activeModifyApply.setAuditUserId(userId);
        activeModifyApply.setAuditUserName(userUtil.getUser().getUserName());
        activeModifyApply.setUpdateBy(userId);
        baseActiveModifyApplyMapper.updateById(activeModifyApply);

        //修改码活动
        active.setActiveName(activeModifyApply.getActiveName());
        active.setActiveType(activeModifyApply.getActiveType());
        active.setBeginTime(activeModifyApply.getBeginTime());
        active.setEndTime(activeModifyApply.getEndTime());
        active.setBrandCode(activeModifyApply.getBrandCode());
        active.setActiveUrl(activeModifyApply.getActiveUrl());
        active.setAuditUserId(userId);
        active.setAuditUserName(userUtil.getUser().getUserName());
        active.setUpdateBy(userId);
        active.setIsModifyApply(0);
        baseActiveMapper.updateById(active);

        //分别查出新增的码包和删除的码包
        //批量新增
        QueryWrapper<BaseActiveModifyApplyQrPackageMapping> activeModifyApplyQrPackageMappingQWAdd = new QueryWrapper<>();
        activeModifyApplyQrPackageMappingQWAdd.eq("apply_id", req.getApplyId());
        activeModifyApplyQrPackageMappingQWAdd.eq("is_delete", 0);
        activeModifyApplyQrPackageMappingQWAdd.eq("modify_type", ActiveModifyTypeEnum.ADD.getCode());
        List<BaseActiveModifyApplyQrPackageMapping> baseActiveModifyApplyQrPackageMappingAdds = baseActiveModifyApplyQrPackageMappingMapper.selectList(activeModifyApplyQrPackageMappingQWAdd);
        if(CollectionUtil.isNotEmpty(baseActiveModifyApplyQrPackageMappingAdds)){
            //聚合新增的码包
            List<Long> packageIdAddList = baseActiveModifyApplyQrPackageMappingAdds.stream().map(BaseActiveModifyApplyQrPackageMapping::getQrPackageId).collect(Collectors.toList());
            baseActiveQrPackageMappingMapper.batchInsert(req.getActiveId(), packageIdAddList, baseActiveModifyApplyQrPackageMappingAdds.get(0).getCreateBy());
        }

        //批量删除
        QueryWrapper<BaseActiveModifyApplyQrPackageMapping> activeModifyApplyQrPackageMappingQWDelete = new QueryWrapper<>();
        activeModifyApplyQrPackageMappingQWDelete.eq("apply_id", req.getApplyId());
        activeModifyApplyQrPackageMappingQWDelete.eq("is_delete", 0);
        activeModifyApplyQrPackageMappingQWDelete.eq("modify_type", ActiveModifyTypeEnum.DELETE.getCode());
        List<BaseActiveModifyApplyQrPackageMapping> baseActiveModifyApplyQrPackageMappingDeletes = baseActiveModifyApplyQrPackageMappingMapper.selectList(activeModifyApplyQrPackageMappingQWAdd);
        if(CollectionUtil.isNotEmpty(baseActiveModifyApplyQrPackageMappingDeletes)){
            //聚合删除的码包
            List<Long> packageIdDeleteList = baseActiveModifyApplyQrPackageMappingDeletes.stream().map(BaseActiveModifyApplyQrPackageMapping::getQrPackageId).collect(Collectors.toList());
            UpdateWrapper<BaseActiveQrPackageMapping> aqpmQW = new UpdateWrapper<>();
            aqpmQW.set("is_delete", 1)
                    .set("update_by", userId)
                    .eq("active_id", req.getActiveId())
                    .in("qr_package_id", packageIdDeleteList);
            baseActiveQrPackageMappingMapper.update(null, aqpmQW);
        }


    }

    @Override
    public void refuseActiveModifyApply(RefuseActiveModifyApplyReq req) {
        //1-查询活动
        BaseActive active = baseActiveMapper.selectById(req.getActiveId());
        AssertUtil.isNull(active,"活动不存在");

        if(!ActiveAuditStatusEnum.REVIEW_PASS.is(active.getAuditStatus())){
            throw new BusinessException("该数据没有审核通过，请调用审核码活动的接口。");
        }
        //查询码活动申请单
        BaseActiveModifyApply activeModifyApply = baseActiveModifyApplyMapper.selectById(req.getApplyId());
        AssertUtil.isNull(activeModifyApply,"码活动申请单不存在");
        if(ActiveAuditStatusEnum.WAIT_REVIEW.is(activeModifyApply.getAuditStatus())){
            throw new BusinessException("该码活动申请单不能审核");
        }

        //审核拒绝码活动申请单
        Long userId = userUtil.getUser().getId();
        activeModifyApply.setAuditStatus(ActiveAuditStatusEnum.REVIEW_REFUSE.getCode());
        activeModifyApply.setAuditUserId(userId);
        activeModifyApply.setAuditUserName(userUtil.getUser().getUserName());
        activeModifyApply.setUpdateBy(userId);
        baseActiveModifyApplyMapper.updateById(activeModifyApply);
    }


    /**
     * 更新码活动和码包关联子表
     * @param activeId
     * @param qrPackageIdAddList
     */
    private void updateActivePackageMappiing(Long activeId, List<Long> qrPackageIdAddList, List<Long> qrPackageIdDeleteList){
        //添加和删除的去重
        List<Long> distinctBatch = new ArrayList<>(qrPackageIdAddList);
        distinctBatch.retainAll(qrPackageIdAddList);
        qrPackageIdAddList.removeAll(distinctBatch);
        qrPackageIdDeleteList.removeAll(distinctBatch);

        Long loginUserId = userUtil.getUser().getId();

        //批量删除
        if(CollectionUtil.isNotEmpty(qrPackageIdDeleteList)){
            UpdateWrapper<BaseActiveQrPackageMapping> aqpmQW = new UpdateWrapper<>();
            aqpmQW.set("is_delete", 1)
                    .set("update_by", loginUserId)
                    .eq("active_id", activeId)
                    .in("qr_package_id", qrPackageIdDeleteList);
            baseActiveQrPackageMappingMapper.update(null, aqpmQW);
        }

        //批量添加
        if(CollectionUtil.isNotEmpty(qrPackageIdAddList)){
            List<QrPackage> qrPackages = searchReviewPassPackageByPackageIds(qrPackageIdAddList);
            List<Long> qrPackageIdsAdd = qrPackages.stream().map(QrPackage::getId).collect(Collectors.toList());
            if(CollectionUtil.isNotEmpty(qrPackageIdsAdd)) {
                baseActiveQrPackageMappingMapper.batchInsert(activeId, qrPackageIdsAdd, loginUserId);
            }
        }
    }

    /**
     * 根据packgeId列表查询有效的且审核通过的码包列表
     * @param packageIdList
     * @return
     */
    private List<QrPackage> searchReviewPassPackageByPackageIds(List<Long> packageIdList){
        QueryWrapper<QrPackage> qrPackageQueryWrapper = new QueryWrapper<>();
        qrPackageQueryWrapper.in("id", packageIdList);
        qrPackageQueryWrapper.eq("apply_status", QrPackageApplyStatusEnum.REVIEW_PASS.getCode());
        qrPackageQueryWrapper.eq("is_delete", 0);
        return qrPackageMapper.selectList(qrPackageQueryWrapper);
    }

    /**
     * 批量插入 活动编辑申请码包关联子表
     * @param activeId
     * @param qrPackageIdAddList
     */
    private void batchInsertActiveModifyApplyPackageMappiing(Long applyId, Long activeId, List<Long> qrPackageIdAddList, List<Long> qrPackageIdDeleteList){
        //添加和删除的去重
        List<Long> distinctBatch = new ArrayList<>(qrPackageIdAddList);
        distinctBatch.retainAll(qrPackageIdDeleteList);
        qrPackageIdAddList.removeAll(distinctBatch);
        qrPackageIdDeleteList.removeAll(distinctBatch);

        List<BatchInsertActiveModifyApplyQrPackageColumnReq.PackageAndModifyType> batchInsertamaqpcrpam = new ArrayList<>();
        Long loginUserId = userUtil.getUser().getId();
        qrPackageIdAddList.forEach(qrPackageIdAdd -> {
            BatchInsertActiveModifyApplyQrPackageColumnReq.PackageAndModifyType packageAndModifyType = new BatchInsertActiveModifyApplyQrPackageColumnReq.PackageAndModifyType();
            packageAndModifyType.setQrPackageId(qrPackageIdAdd);
            packageAndModifyType.setModifyType(ActiveModifyTypeEnum.ADD.getCode());
            batchInsertamaqpcrpam.add(packageAndModifyType);
        });

        qrPackageIdDeleteList.forEach(qrPackageIdDelete -> {
            BatchInsertActiveModifyApplyQrPackageColumnReq.PackageAndModifyType packageAndModifyType = new BatchInsertActiveModifyApplyQrPackageColumnReq.PackageAndModifyType();
            packageAndModifyType.setQrPackageId(qrPackageIdDelete);
            packageAndModifyType.setModifyType(ActiveModifyTypeEnum.DELETE.getCode());
            batchInsertamaqpcrpam.add(packageAndModifyType);
        });

        //批量插入
        BatchInsertActiveModifyApplyQrPackageColumnReq columnReq = new BatchInsertActiveModifyApplyQrPackageColumnReq();
        columnReq.setApplyId(applyId);
        columnReq.setActiveId(activeId);
        columnReq.setCreateBy(loginUserId);
        columnReq.setUpdateBy(loginUserId);
        columnReq.setPackageAndModifyTypeList(batchInsertamaqpcrpam);
        baseActiveModifyApplyQrPackageMappingMapper.batchInsert(columnReq);
    }

}
