package com.abi.qms.platform.service.impl;

import cn.hutool.core.collection.CollectionUtil;
import com.abi.base.foundation.util.RedisClient;
import com.abi.qms.platform.constant.RedisKeysConstant;
import com.abi.qms.platform.dao.entity.BaseFactory;
import com.abi.qms.platform.dao.entity.QrPackage;
import com.abi.qms.platform.dao.entity.QrPackageBatch;
import com.abi.qms.platform.dao.entity.QrPackageDownloadRecord;
import com.abi.qms.platform.dao.entity.QrRepertoryColumn;
import com.abi.qms.platform.dao.entity.UserInfo;
import com.abi.qms.platform.dao.enums.*;
import com.abi.qms.platform.dao.mapper.*;
import com.abi.qms.platform.dao.entity.*;
import com.abi.qms.platform.dao.enums.FactoryLevelEnum;
import com.abi.qms.platform.dao.enums.FactoryTypeEnum;
import com.abi.qms.platform.dao.enums.InvalidEnum;
import com.abi.qms.platform.dao.enums.QrPackageApplyStatusEnum;
import com.abi.qms.platform.dao.enums.QrPackageGenerateStatusEnum;
import com.abi.qms.platform.dao.enums.QrRepertoryTypeEnum;
import com.abi.qms.platform.dao.mapper.*;
import com.abi.qms.platform.dao.tablestore.entity.QrCode;
import com.abi.qms.platform.dao.vo.result.*;
import com.abi.qms.platform.dto.req.*;
import com.abi.qms.platform.dto.res.*;
import com.abi.qms.platform.infrastructure.mq.GenerateCodeConsumer;
import com.abi.qms.platform.infrastructure.util.AssertUtil;
import com.abi.qms.platform.infrastructure.util.FreeMarkerUtils;
import com.abi.qms.platform.infrastructure.util.OssFileDownloadUtil;
import com.abi.qms.platform.infrastructure.util.PageUtil;
import com.abi.qms.platform.infrastructure.util.RandomCodeUtils;
import com.abi.qms.platform.infrastructure.util.SendmailUtil;
import com.abi.qms.platform.infrastructure.util.SmsUtil;
import com.abi.qms.platform.infrastructure.util.UserUtil;
import com.abi.qms.platform.service.QrPackageService;
import com.abi.task.common.api.exception.BusinessException;
import com.abi.task.common.api.exception.ErrorCodeEnum;
import com.abi.task.common.tablestore.TableStorePlusUtils;
import com.abi.task.common.tablestore.TableStoreUtils;
import com.abi.task.common.utils.PojoConverterUtils;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import org.apache.commons.collections.CollectionUtils;
import freemarker.template.Template;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.ObjectUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.amqp.core.AmqpTemplate;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.ui.freemarker.FreeMarkerTemplateUtils;

import javax.servlet.http.HttpServletResponse;
import java.text.MessageFormat;
import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.concurrent.TimeUnit;
import java.util.stream.Collectors;

/**
 * 码包 Service业务层处理
 *
 * @author WeiganCai
 * @date 2021-04-28
 */
@Service
@Slf4j
public class QrPackageServiceImpl implements QrPackageService {

    @Autowired
    private QrPackageMapper qrPackageMapper;

    @Autowired
    private BaseFactoryMapper baseFactoryMapper;

    @Autowired
    private QrPackageDownloadRecordMapper qrPackageDownloadRecordMapper;

    @Autowired
    private QrPackageBatchMapper qrPackageBatchMapper;

    @Autowired
    private AmqpTemplate amqpTemplate;

    @Autowired
    private SendmailUtil sendmailUtil;

    @Autowired
    private SmsUtil smsUtil;

    @Autowired
    private RedisClient redisClient;

    @Autowired
    private UserUtil userUtil;

    @Autowired
    private TableStorePlusUtils tableStorePlusUtils;

    @Autowired
    private QrRepertoryColumnMapper repertoryColumnMapper;

    @Autowired
    private BaseActiveMapper baseActiveMapper;

    @Autowired
    private QrBoxMappingMapper qrBoxMappingMapper;

    @Autowired
    private TableStoreUtils tableStoreUtils;

    @Autowired
    private BaseMaterialMapper baseMaterialMapper;
    @Autowired
    private BaseBrandMapper baseBrandMapper;

    /**
     * 保存码包
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public void saveQrPackage(SaveQrPackageReq req) {
        boolean isAdd = (req.getId() == null);

        //0-校验入参
        QrPackage qrPackage = checkRequestWhenSaveQrPackage(req);

        //1-自动补全or处理入参数据
        fullParam(req);

        //2-构造码包主表对象
        qrPackage = buildQrPackage(req, qrPackage);

        //3-新增or修改 码包主表
        if (!isAdd) {
            qrPackageMapper.updateById(qrPackage);
        } else {
            qrPackageMapper.insert(qrPackage);
        }

        //4-保存码包批次子表信息
        saveQrPackageBatch(qrPackage, isAdd, req);

    }

    /**
     * 自动补全or处理入参数据
     */
    private void fullParam(SaveQrPackageReq req) {
        //1-如果没有传入二级供应商，则表示选的是无等级（即自己做自己）的包材厂，那么就把自己放入二级供应商内
        if (req.getFactoryCoverLevelTwoId() == null) {
            req.setFactoryCoverLevelTwoId(req.getFactoryCoverId());
        }

        //2-如果批次信息为空（盖码 or 真实业务没有批次的箱码包材厂），则补充批次信息
        if (CollectionUtil.isEmpty(req.getPackageBatchBeanList())) {
            //列表
            List<SaveQrPackageReq.PackageBatchBean> batchBeanList = new ArrayList<>();

            //码的总量
            Long qrNumber = req.getQrNumber();

            //批次信息按包材厂的最大文件数来拆分
            BaseFactory factory = baseFactoryMapper.selectById(req.getFactoryCoverLevelTwoId());
            Long maxFileCount = factory.getMaxFileCount();
            //如果二级包材厂的最大数为空，则认为和总数一致
            if (maxFileCount == null || maxFileCount <= 0) {
                maxFileCount = qrNumber;
            }

            //计算需要生成多少批次，不是整除则+1
            int batchCount = qrNumber.intValue() / maxFileCount.intValue();
            if (batchCount * maxFileCount < qrNumber) {
                batchCount = batchCount + 1;
            }

            //循环添加批次
            for (int i = 0; i < batchCount; i++) {
                //每个批次多少条
                Long batchQrNumber = maxFileCount;
                if (i == batchCount - 1) {
                    batchQrNumber = qrNumber - (batchCount - 1) * maxFileCount;
                }
                SaveQrPackageReq.PackageBatchBean batchBean = new SaveQrPackageReq.PackageBatchBean();
                batchBean.setBatchNumber("000001");
                batchBean.setBatchQrNumber(req.getQrNumber());
                batchBeanList.add(batchBean);
            }
            req.setPackageBatchBeanList(batchBeanList);
        }

        //3-如果是盖码，那么查询盖码得默认箱码幅面，并放入入参
        //TODO


    }

    /**
     * 保存码包批次子表信息
     *
     * @param qrPackage
     * @param isAdd
     * @param req
     */
    private void saveQrPackageBatch(QrPackage qrPackage, boolean isAdd, SaveQrPackageReq req) {
        //删除批次号（仅更新码包可能删除）
        if (!isAdd) {
            //查询原有批次号
            QueryWrapper<QrPackageBatch> qrPackageBatchQuery = new QueryWrapper<>();
            qrPackageBatchQuery.eq("package_id", qrPackage.getId());
            qrPackageBatchQuery.eq("is_delete", 0);
            List<QrPackageBatch> qrPackageBatches = qrPackageBatchMapper.selectList(qrPackageBatchQuery);
            List<Long> batchOldIds = qrPackageBatches.stream().map(QrPackageBatch::getId).collect(Collectors.toList());
            //更新的批次数据
            List<Long> batchNewIds = req.getPackageBatchBeanList().stream().map(SaveQrPackageReq.PackageBatchBean::getId).collect(Collectors.toList());
            //原有-入参更新 = 删除的数据
            List<Long> deleteBatch = new ArrayList<>(batchOldIds);
            deleteBatch.removeAll(batchNewIds);
            if (CollectionUtil.isNotEmpty(deleteBatch)) {
                UpdateWrapper<QrPackageBatch> qrPackageBatchDeleteParam = new UpdateWrapper<>();
                qrPackageBatchDeleteParam.in("id", deleteBatch)
                        .set("is_delete", 1);
                qrPackageBatchMapper.update(null, qrPackageBatchDeleteParam);
            }
        }

        //添加或更新
        for (SaveQrPackageReq.PackageBatchBean packageBatchBean : req.getPackageBatchBeanList()) {
            if (packageBatchBean.getId() == null) {
                QrPackageBatch qrPackageBatch = new QrPackageBatch();
                qrPackageBatch.setPackageId(qrPackage.getId());
                qrPackageBatch.setBatchNumber(packageBatchBean.getBatchNumber());
                qrPackageBatch.setBatchQrNumber(packageBatchBean.getBatchQrNumber());
                qrPackageBatchMapper.insert(qrPackageBatch);
            } else {
                QueryWrapper<QrPackageBatch> qrPackageBatchQ = new QueryWrapper<>();
                qrPackageBatchQ.eq("id", packageBatchBean.getId());
                qrPackageBatchQ.eq("is_delete", 0);
                QrPackageBatch qrPackageBatch = qrPackageBatchMapper.selectOne(qrPackageBatchQ);
                if (Objects.isNull(qrPackageBatch)) {
                    throw new BusinessException(ErrorCodeEnum.NO_EXISTS_DATA.getCode(), "批次对象不存在");
                }
                qrPackageBatch.setBatchQrNumber(packageBatchBean.getBatchQrNumber());
                qrPackageBatch.setBatchNumber(packageBatchBean.getBatchNumber());
                qrPackageBatchMapper.updateById(qrPackageBatch);
            }
        }
    }


    /**
     * build数据库的码包对象
     *
     * @param req
     * @param qrPackage
     * @return
     */
    private QrPackage buildQrPackage(SaveQrPackageReq req, QrPackage qrPackage) {
        if (qrPackage == null) {
            qrPackage = new QrPackage();
        }

        qrPackage.setSapOrderNo(req.getSapOrderNo());
        qrPackage.setMaterialId(req.getMaterialId());
        qrPackage.setFactoryCoverId(req.getFactoryCoverId());
        qrPackage.setFactoryCoverLevelTwoId(req.getFactoryCoverLevelTwoId());
        qrPackage.setFactoryBeerId(req.getFactoryBeerId());
        qrPackage.setBoxCodeFormatId(req.getBoxCodeFormatId());
        qrPackage.setQrNumber(req.getQrNumber());
        qrPackage.setStandbyRatio(req.getStandbyRatio());
        qrPackage.setReasonApply(req.getReasonApply());
        qrPackage.setInvalid(InvalidEnum.NOT_INVALID.getCode());
        //sta 根据物料ID查询物料类型，判断码类型，保存至码包表中
        GetPackageCodeReq Material=new GetPackageCodeReq();
        Material.setId(req.getMaterialId());
        MaterialVO vo=baseMaterialMapper.getPackageCodeType(Material);
        AssertUtil.isNull(vo, "物料信息不存在");
        qrPackage.setQrType(getPackageCode(vo.getMaterialType()));
        //end
        return qrPackage;
    }

    /**
     * 校验保存码包的入参
     *
     * @param req
     * @return QrPackage 如果是修改码包就把码包查出来
     */
    private QrPackage checkRequestWhenSaveQrPackage(SaveQrPackageReq req) {
        QrPackage qrPackage = null;
        Long qrPackageId = req.getId();
        boolean isAdd = (req.getId() == null);

        // 修改码包时的条件限制
        if (qrPackageId != null) {
            qrPackage = qrPackageMapper.selectById(qrPackageId);
            AssertUtil.isNull(qrPackage, "码包不存在");
            // 只有待审核、审核拒绝状态下才可编辑
            Integer applyStatus = qrPackage.getApplyStatus();
            if (QrPackageApplyStatusEnum.REVIEW_PASS.is(applyStatus)) {
                throw new BusinessException("码包不可编辑");
            }
        }
        //如果SAP订单号和物料有重复数据，需提交额外申请原因
        ValidSAPOrderNoAndMaterialReq reqValid = new ValidSAPOrderNoAndMaterialReq();
        reqValid.setMaterialId(req.getMaterialId());
        reqValid.setSapOrderNo(req.getSapOrderNo());
        ValidSAPOrderNoAndMaterialRes validSAPOrderNoAndMaterialRes = validSAPOrderNoAndMaterial(reqValid);
        if (Objects.nonNull(validSAPOrderNoAndMaterialRes) && CollectionUtil.isNotEmpty(validSAPOrderNoAndMaterialRes.getPackageBeanList())) {
            Long id = validSAPOrderNoAndMaterialRes.getPackageBeanList().get(0).getId();
            if (validSAPOrderNoAndMaterialRes.getPackageBeanList().size() > 1) {
                if (Objects.isNull(req.getReasonApply())) {
                    throw new BusinessException("该SAP订单号和物料与码包" + id + "重复，若仍需申请需要提交额外码包申请原因");
                }
            }
            if (validSAPOrderNoAndMaterialRes.getPackageBeanList().size() == 1) {
                if (req.getId() == null) {
                    if (Objects.isNull(req.getReasonApply())) {
                        throw new BusinessException("该SAP订单号和物料与码包" + id + "重复，若仍需申请需要提交额外码包申请原因");
                    }
                }
                if (!id.equals(req.getId())) {
                    if (Objects.isNull(req.getReasonApply())) {
                        throw new BusinessException("该SAP订单号和物料与码包" + id + "重复，若仍需申请需要提交额外码包申请原因");
                    }
                }
            }
        }

        //若当前登录用户为包材厂用户，则：
        UserInfo wholeUser = userUtil.getWholeUser();
        BaseFactory factoryCover = baseFactoryMapper.selectById(req.getFactoryCoverId());
        //如果选择的是一级包材厂（而不是无等级包材厂），那么二级包材厂必填
        if (FactoryLevelEnum.ONE.is(factoryCover.getFactoryLevel())
                && req.getFactoryCoverLevelTwoId() == null) {
            throw new BusinessException("选择了一级包材厂，则二级包材厂必填");
        }
        if (wholeUser.getFactoryId() != null && FactoryTypeEnum.COVER.is(wholeUser.getFactoryType())) {
            //1.他不是二级包材厂，包材厂必须为该用户所在的包材厂，不可修改
            if (!FactoryLevelEnum.TWO.is(factoryCover.getFactoryLevel())
                    && !req.getFactoryCoverId().equals(wholeUser.getFactoryId())) {
                throw new BusinessException("包材厂只能选择登录用户的包材厂");
            }
            //2.他是二级包材厂，二级包菜场必须为用户所在的包材厂，不可修改
            if (FactoryLevelEnum.TWO.is(factoryCover.getFactoryLevel())
                    && !req.getFactoryCoverLevelTwoId().equals(wholeUser.getFactoryId())) {
                throw new BusinessException("二级包材厂只能选择登录用户的包材厂");
            }
        }

        if (!isAdd && wholeUser.getFactoryId() != null) {
            //一级包材厂不允许修改码包
            if (FactoryLevelEnum.ONE.is(factoryCover.getFactoryLevel())) {
                throw new BusinessException("一级包材厂不允许修改码包");
            }
            //二级包材厂or无级包材厂，只能改自己得码包
            if (!FactoryLevelEnum.ONE.is(factoryCover.getFactoryLevel())
                    && !wholeUser.getFactoryId().equals(qrPackage.getFactoryCoverLevelTwoId())) {
                throw new BusinessException("包材厂只允许修改自己的码包");
            }
        }

        //校验批次号
        if (CollectionUtil.isNotEmpty(req.getPackageBatchBeanList())) {
            //同一个码包内，不同批次的批次号不一样
            List<SaveQrPackageReq.PackageBatchBean> packageBatchBeanList = req.getPackageBatchBeanList();
            //聚合批次号
            List<String> batchNumberList = packageBatchBeanList.stream().map(SaveQrPackageReq.PackageBatchBean::getBatchNumber).distinct().collect(Collectors.toList());
            if (batchNumberList.size() < req.getPackageBatchBeanList().size()) {
                throw new BusinessException("批次号重复，请确认后重试");
            }
            //同一个码包内，批次加起来的总数量和码包数量一致
            //计算码包数量总和
            long sum = packageBatchBeanList.stream().mapToLong(SaveQrPackageReq.PackageBatchBean::getBatchQrNumber).sum();
            if (sum != req.getQrNumber()) {
                throw new BusinessException("批次码数量与申请码总量不匹配，请确认后重试");
            }
        }

        return qrPackage;
    }


    /**
     * 码包分页查询
     */
    @Override
    public ListQrPackageRes listQrPackage(ListQrPackageReq req) {
        //数据权限隔离，如果登录账号是包材厂，只能看账号包材厂的数据。否则看全部
        if (userUtil.getWholeUser().getFactoryId() != null && userUtil.getWholeUser().getFactoryType() != null && FactoryTypeEnum.COVER.is(userUtil.getWholeUser().getFactoryType())) {
            req.setFactoryCoverId(userUtil.getWholeUser().getFactoryId());
        }

        // 分页查询
        IPage<QrPackageVO> iPage = qrPackageMapper.listQrPackage(PageUtil.createPage(req), req);
        List<QrPackageVO> qrPackageList = iPage.getRecords();

        // 封装出参、放入分页信息
        ListQrPackageRes res = new ListQrPackageRes();
        PageUtil.copyPageInfo(res, iPage);
        List<ListQrPackageRes.QrPackageBean> qrPackageBeanList = PojoConverterUtils.copyList(qrPackageList, ListQrPackageRes.QrPackageBean.class);
        res.setQrRepertoryBeanList(qrPackageBeanList);

        return res;
    }

    /**
     * 码包查询详情
     */
    @Override
    public GetQrPackageDetailRes getQrPackageDetail(GetQrPackageDetailReq req) {
        // 查询码包对象
        QrPackageVO qrPackage = qrPackageMapper.selectQrPackageDetailById(req.getId());
        AssertUtil.isNull(qrPackage, "码包不存在");

        //转化出参
        GetQrPackageDetailRes res = PojoConverterUtils.copy(qrPackage, GetQrPackageDetailRes.class);

        //查询码包对应活动单条数据
        PackageActiveVO packageActive = baseActiveMapper.getPackageActive(qrPackage.getId());

        //存在活动数据
        if (ObjectUtils.isNotEmpty(packageActive)) {
            getBrandCodeName(res,packageActive);
        }
        return res;
    }

    /**
     * 活动详情封装出参，显示品牌名称
     * @param res
     * @param packageActive
     */
    private void getBrandCodeName(GetQrPackageDetailRes res,PackageActiveVO packageActive){

        res.setActiveCode(packageActive.getActiveCode());
        res.setActiveName(packageActive.getActiveName());
        res.setActiveUrl(packageActive.getActiveUrl());
        res.setActiveType(packageActive.getActiveType());
        res.setBeginTime(packageActive.getBeginTime());
        res.setEndTime(packageActive.getEndTime());
        //多品牌名称
        StringBuffer brandNames = new StringBuffer();
        //查询品牌一级列表
        QueryWrapper<BaseBrand> baseBrandQW = new QueryWrapper<>();
        baseBrandQW.eq("brand_level",1);
        List<BaseBrand> baseBrands = baseBrandMapper.selectList(baseBrandQW);
        Map<String, String> brandCodeMap = baseBrands.stream().collect(Collectors.toMap(BaseBrand::getBrandCode, BaseBrand::getBrandName));

        //品牌名称获取
        String[] brandCodeArray = packageActive.getBrandCode().split(",");
        for (String brandCode : brandCodeArray) {
            String codeName = brandCodeMap.get(brandCode);
            if (StringUtils.isNotBlank(codeName)) {
                brandNames.append(codeName).append(",");
            }
        }

        if (brandNames.length()>0){
            brandNames.deleteCharAt(brandNames.length()-1);
            res.setBrandNames(brandNames.toString());
        }
    }

    /**
     * 删除码包
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public void deleteQrPackage(DeleteQrPackageReq req) {
        QrPackage qrPackage = qrPackageMapper.selectById(req.getId());
        AssertUtil.isNull(qrPackage, "码包不存在");
        // 只有待审核、审核拒绝状态下才可删除
        Integer applyStatus = qrPackage.getApplyStatus();
        if (QrPackageApplyStatusEnum.REVIEW_PASS.is(applyStatus)) {
            throw new BusinessException("码包不可删除");
        }

        // 逻辑删除
        QrPackage update = new QrPackage().setId(req.getId()).setIsDelete(1);
        qrPackageMapper.updateById(update);
    }

    /**
     * 审核通过码包
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public void passQrPackage(PassQrPackageReq req) {
        QrPackageVO qrPackage = qrPackageMapper.selectQrPackageDetailById(req.getId());
        AssertUtil.isNull(qrPackage, "码包不存在");

        // 只有待审核、审核拒绝状态下才可通过
        Integer applyStatus = qrPackage.getApplyStatus();
        if (QrPackageApplyStatusEnum.REVIEW_PASS.is(applyStatus)) {
            throw new BusinessException("码包不可通过");
        }

        QrPackage update = new QrPackage().setId(req.getId()).setApplyStatus(QrPackageApplyStatusEnum.REVIEW_PASS.getCode());
        qrPackageMapper.updateById(update);

        // 码库类型为系统生成时，放入rabbitmq，生成码
        if (QrRepertoryTypeEnum.SYSTEM_GENERATE.is(qrPackage.getQrRepertoryType())) {
            amqpTemplate.convertAndSend(GenerateCodeConsumer.GENERATE_CODE_BY_REPERTORY_QUEUE, req.getId());
        }
    }

    /**
     * 审核拒绝码包
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public void refuseQrPackage(RefuseQrPackageReq req) {
        QrPackage qrPackage = qrPackageMapper.selectById(req.getId());
        AssertUtil.isNull(qrPackage, "码包不存在");

        // 只有待审核、审核拒绝状态下才可拒绝
        Integer applyStatus = qrPackage.getApplyStatus();
        if (QrPackageApplyStatusEnum.REVIEW_PASS.is(applyStatus)) {
            throw new BusinessException("码包不可拒绝");
        }

        QrPackage update = new QrPackage().setId(req.getId()).setApplyStatus(QrPackageApplyStatusEnum.REVIEW_REFUSE.getCode());
        qrPackageMapper.updateById(update);
    }

    @Override
    public void downloadQrPackage(DownloadQrPackageReq req, HttpServletResponse response) {
        //查询码包
        QrPackage qrPackage = qrPackageMapper.selectById(req.getId());
        AssertUtil.isNull(qrPackage, "码包不存在");
        //查询包材厂
        BaseFactory factory = baseFactoryMapper.selectById(qrPackage.getFactoryCoverId());
        AssertUtil.isNull(factory, "包材厂不存在");
        if (StringUtils.isBlank(factory.getKeymanMobile())) {
            throw new BusinessException("包材厂手机号不存在");
        }

        // 只有已生成状态下才可下载
        Integer generateStatus = qrPackage.getGenerateStatus();
        if (!QrPackageGenerateStatusEnum.GENERATE_SUCCESS.is(generateStatus)) {
            throw new BusinessException("码包不可下载");
        }

        //特殊：校验验证码是否正确，正确才可以下载
        boolean verifyCaptchaResult = verifyCaptcha(factory.getKeymanMobile(), req.getCaptcha(), qrPackage.getId());
        if (!verifyCaptchaResult) {
            throw new BusinessException("验证码错误");
        }

        //开始下载
        String url = qrPackage.getDownloadPath();
        try {
            new OssFileDownloadUtil().downloadFile(url, response);
        } catch (Exception e) {
            log.error("下载失败", e);
            throw new BusinessException("下载失败");
        }

        //插入码包下载记录
        Long userId = userUtil.getUser().getId();
        QrPackageDownloadRecord downloadRecord = new QrPackageDownloadRecord()
                .setCreateBy(userId)
                .setQrPackageId(qrPackage.getId());
        qrPackageDownloadRecordMapper.insert(downloadRecord);

    }

    /**
     * 码生成失败回滚
     */
    @Override
    public void failedRollback(QrPackage qrPackage) {
        // 更改生成状态为生成失败
        QrPackage qrPackageUpdate = new QrPackage().setId(qrPackage.getId()).setGenerateStatus(QrPackageGenerateStatusEnum.GENERATE_FAIL.getCode());
        qrPackageMapper.updateById(qrPackageUpdate);
    }

    /**
     * 校验是否存在SAP订单号和物料重复数据
     *
     * @param req
     * @return
     */
    @Override
    public ValidSAPOrderNoAndMaterialRes validSAPOrderNoAndMaterial(ValidSAPOrderNoAndMaterialReq req) {
        //查询存在的列表
        QueryWrapper<QrPackage> qrPackageQuery = new QueryWrapper<>();
        qrPackageQuery.eq("material_id", req.getMaterialId());
        qrPackageQuery.eq("sap_order_no", req.getSapOrderNo());
        qrPackageQuery.eq("is_delete", 0);
        List<QrPackage> qrPackages = qrPackageMapper.selectList(qrPackageQuery);

        //构造出参
        ValidSAPOrderNoAndMaterialRes res = new ValidSAPOrderNoAndMaterialRes();
        List<ValidSAPOrderNoAndMaterialRes.PackageBean> packageBeanList = PojoConverterUtils.copyList(qrPackages, ValidSAPOrderNoAndMaterialRes.PackageBean.class);
        res.setPackageBeanList(packageBeanList);

        return res;
    }

    /**
     * 码包生成完成后发送邮件给码包厂负责人邮箱
     *
     * @param targetEmail
     * @param sapOrderNo
     * @param filename
     * @param password
     */
    @Override
    public void sendEmailForQrPackageCreated(String targetEmail, String sapOrderNo, String filename, String password) {
        try {
            //拿到freemarker模板
            Template template = FreeMarkerUtils.getTemplate("factory.ftl");

            //填充的内容
            Map paramMap = new HashMap<>();
            paramMap.put("sapOrderNo", sapOrderNo);
            paramMap.put("filename", filename);
            paramMap.put("password", password);

            //生成需要发送的内容
            String emailContent = FreeMarkerTemplateUtils.processTemplateIntoString(template, paramMap);

            sendmailUtil.doSendHtmlEmail("QMS系统通知", emailContent, targetEmail);

        } catch (Exception e) {
            log.info("发送码包通知邮件异常", e);
            throw new BusinessException("发送码包通知邮件异常");
        }
    }

    /**
     * 发送短信验证码
     *
     * @param req
     */
    @Override
    public void sendMobileMsg(SendMobileMsgReq req) {
        //查询码包
        QrPackage qrPackage = qrPackageMapper.selectById(req.getId());
        AssertUtil.isNull(qrPackage, "码包不存在");
        //查询包材厂
        BaseFactory factory = baseFactoryMapper.selectById(qrPackage.getFactoryCoverId());
        AssertUtil.isNull(factory, "包材厂不存在");
        if (StringUtils.isBlank(factory.getKeymanMobile())) {
            throw new BusinessException("包材厂手机号不存在");
        }

        //key:packId:mobile
        Long packId = qrPackage.getId();
        String mobile = factory.getKeymanMobile();
        String key = MessageFormat.format("{0}{1}:{2}", RedisKeysConstant.DOWNLOAD_MOBILE_MSG, packId, mobile);

        //获取随机验证码
        String captcha = RandomCodeUtils.getCaptcha(6);

        //发送短信
        smsUtil.sendCode(mobile, captcha);

        //将验证码存入redis
        redisClient.getRedisTemplate().opsForValue().set(key, captcha, 30 * 60, TimeUnit.SECONDS);
    }

    /**
     * 校验校验码
     *
     * @param mobile
     * @param inputCaptcha
     * @param packId
     * @return
     */
    private boolean verifyCaptcha(String mobile, String inputCaptcha, Long packId) {
        String key = MessageFormat.format("{0}{1}:{2}", RedisKeysConstant.DOWNLOAD_MOBILE_MSG, packId, mobile);
        log.info("验证下载key:{}", key);
        String captcha = redisClient.get(key);
        log.info(inputCaptcha + "--------------------" + captcha);

        //不一致返回false
        if (!inputCaptcha.equals(captcha)) {
            return false;
        }

        //删除校验码
        redisClient.delete(key);

        return true;
    }

    @Override
    public GetQrSingleCheckDetailRes getQrSingleCheckDetail(GetQrSingleCheckDetailCodeReq req) {

        //调用tablestore查询码信息
        QrCode qrCode = tableStorePlusUtils.selectOne(QrCode.class, req.getQrCode());
        AssertUtil.isNull(qrCode.getPackageId(), "码数据不存在");

        //1-查询码关联信息
        QrSingleCheckVO qrSingleCheckVO = qrPackageMapper.getQrSingleCheckDetailById(Long.valueOf(qrCode.getPackageId()));
        AssertUtil.isNull(qrSingleCheckVO, "码数据不存在");

        //查询是否存在隐形码
        QrRepertoryColumn qrRepertoryColumn = repertoryColumnMapper.selectById(qrCode.getQrRepertoryColumnId());

        //存在别名（隐形码）
        if (ObjectUtils.isNotEmpty(qrRepertoryColumn)) {
            qrSingleCheckVO.setAlias(qrRepertoryColumn.getAlias());//隐形码
        }

        //查询码包对应活动单条数据
        PackageActiveVO packageActive = baseActiveMapper.getPackageActive(qrCode.getPackageId());

        //存在活动数据
        if (ObjectUtils.isNotEmpty(packageActive)) {
            qrSingleCheckVO.setActiveName(packageActive.getActiveName());
            qrSingleCheckVO.setActiveUrl(packageActive.getActiveUrl());
        }

        //查询批次号id
        QrOnePackageBatchReq qrOnePackageBatchReq = new QrOnePackageBatchReq();
        qrOnePackageBatchReq.setPackageId(qrCode.getPackageId());
        qrOnePackageBatchReq.setBatchNumber(qrCode.getBatchNumberId());
        QrPackageBatchVO qrPackageBatchOne = qrPackageBatchMapper.getQrPackageBatchOne(qrOnePackageBatchReq);

        //存批次号
        if (ObjectUtils.isNotEmpty(qrPackageBatchOne)) {
            qrSingleCheckVO.setBatchNumber(qrPackageBatchOne.getBatchNumber());
        }

        //1-查询码关联激活信息单条数据
        QrOneBoxCodeMappingReq qrOneBoxCodeMappingReq = new QrOneBoxCodeMappingReq();
        qrOneBoxCodeMappingReq.setPackageId(qrCode.getPackageId());
        qrOneBoxCodeMappingReq.setQrIndex(qrCode.getCodeIndex());
        QrBoxMappingVO qrBoxMappingVO = qrBoxMappingMapper.getIndexQrBoxCodeMappingOne(qrOneBoxCodeMappingReq);

        //存在激活关联信息
        if (ObjectUtils.isNotEmpty(qrBoxMappingVO)) {
            qrSingleCheckVO.setBoxCode(qrBoxMappingVO.getBoxCode());//条码序号
            qrSingleCheckVO.setActiveTime(qrBoxMappingVO.getActiveTime());//激活时间
            qrSingleCheckVO.setActiveUserName(qrBoxMappingVO.getActiveUserName());//激活时间
            qrSingleCheckVO.setActiveStatus(qrBoxMappingVO.getActiveStatus());//激活状态
        }

        //转化出参
        GetQrSingleCheckDetailRes res = PojoConverterUtils.copy(qrSingleCheckVO, GetQrSingleCheckDetailRes.class);
        res.setQrCode(qrCode.getCode());
        return res;
    }

    @Override
    public void invalidQrPackage(InvalidQrPackageReq req) {

        //获取登录用户信息
        UserInfo user = userUtil.getUser();

        QrPackage qrPackage = qrPackageMapper.selectById(req.getId());
        AssertUtil.isNull(qrPackage, "该码包不存在！");

        qrPackage.setInvalid(InvalidEnum.INVALID.getCode());
        qrPackage.setUpdateBy(user.getId());
        qrPackage.setUpdateTime(LocalDateTime.now());

        qrPackageMapper.updateById(qrPackage);

    }

    @Override
    public void invalidQrSingle(InvalidQrSingleReq req) {


        try {
            //设置主键code 表名 列名称
            String code = "code", tableName = "qr_code", columnName = "invalid";

            //根据主键code更新列字段
            tableStoreUtils.updateRow(code, req.getCode(), tableName, columnName, InvalidEnum.INVALID.getCode().toString());

        } catch (Exception e) {
            log.info("更新tableStore异常", e);
            throw new BusinessException("更新tableStore异常");
        }
    }
    @Override
    public GetPackageCodeRes getPackageCodeType(GetPackageCodeReq req){
        //判断ID是否为空
        AssertUtil.isNull(req.getId(),"物料ID为空");
        //查询物料类型及suk名称
        MaterialVO materialVO=baseMaterialMapper.getPackageCodeType(req);
        AssertUtil.isNull(materialVO, "物料不存在");
        //转化出参
        GetPackageCodeRes res = PojoConverterUtils.copy(materialVO, GetPackageCodeRes.class);
        //码类型赋值
        res.setQrType(getPackageCode(materialVO.getMaterialType()));
        return res;
    }

    /**
     * 根据物料类型转换码类型
     * 物料类型： 4-瓶盖 5-纸板箱
     * 码类型：1-箱码 2-盖码
     * @param type
     * @return
     */
    public Integer getPackageCode(Integer type){
        if (MaterialTypeEnum.BOTTLE_CAP.is(type)) {
            return QrTypeEnum.CAP.getCode();
        }else{
            return  QrTypeEnum.CARTON.getCode();
        }

    }
}
